@@ -21,8 +21,7 @@ time during install.
 
 """
 
-from HTMLParser import HTMLParser
-from itertools import izip
+from html.parser import HTMLParser
 import io
 import os
 import re
@@ -30,12 +29,6 @@ from tokenize import generate_tokens, COMMENT, NAME, OP, STRING
 
 from jinja2.ext import babel_extract as jinja2_extractor
 
-try:
-    import genshi
-    from genshi.filters.i18n import extract as genshi_extractor
-except ImportError:
-    genshi = None
-
 from distutils import log as distlog
 from distutils.cmd import Command
 from distutils.command.build import build as _build
@@ -101,9 +94,6 @@ try:
     from babel.support import Translations
     from babel.util import parse_encoding
 
-    _GENSHI_MARKUP_SEARCH = re.compile(r'\[[0-9]+:').search
-
-
     _DEFAULT_KWARGS_MAPS = {
         'Option': {'doc': 4},
         'BoolOption': {'doc': 4},
@@ -202,7 +192,7 @@ try:
                     else:
                         messages.append(None)
 
-                    for name, message in messages_kwargs.iteritems():
+                    for name, message in messages_kwargs.items():
                         if name not in func_kwargs_map:
                             continue
                         index = func_kwargs_map[name]
@@ -239,7 +229,7 @@ try:
                     # aid=617979&group_id=5470
                     value = eval('# coding=%s\n%s' % (encoding, value),
                                  {'__builtins__':{}}, {})
-                    if isinstance(value, str):
+                    if isinstance(value, bytes):
                         value = value.decode(encoding)
                     buf.append(value)
                 elif tok == OP and value == '=' and prev_tok == NAME:
@@ -282,56 +272,37 @@ try:
             return []
         out = io.StringIO()
         extractor = ScriptExtractor(out)
-        extractor.feed(unicode(fileobj.read(), 'utf-8'))
+        extractor.feed(str(fileobj.read(), 'utf-8'))
         extractor.close()
         out.seek(0)
         return extract_javascript(out, keywords, comment_tags, options)
 
 
-    def extract_html(fileobj, keywords, comment_tags, options, text=False):
+    def extract_html(fileobj, keywords, comment_tags, options):
         """Extracts translatable texts from templates.
 
         We simplify white-space found in translatable texts collected
         via the ``gettext`` function (which is what the ``trans``
-        directives use) and for text in the legacy Genshi templates,
-        otherwise we would have near duplicates (e.g. admin.html,
-        prefs.html).
+        directives use), otherwise we would have near duplicates
+        (e.g. admin.html, prefs.html).
 
         We assume the template function ``gettext`` will do the same
         before trying to fetch the translation from the catalog.
 
         """
-        extractor = None
-        # TODO (1.5.1) remove genshi support
         if fileobj:
-            if 'xmlns:py="http://genshi.edgewall.org/"' in fileobj.read():
-                if not genshi:
-                    raise ValueError("Cannot extract messages from a legacy "
-                                     "Genshi template because Genshi is "
-                                     "not installed.")
-                if text:
-                    options.update(template_class=
-                                   'genshi.template:NewTextTemplate')
-                extractor = genshi_extractor
-            else:
-                extractor = jinja2_extractor
+            extractor = jinja2_extractor
             fileobj.seek(0)
             for m in extractor(fileobj, keywords, comment_tags, options):
                 # lineno, func, message, comments = m
                 if m[1] in ('gettext', None):
-                    #   (Jinja2 trans, Genshi)
+                    # Jinja2 trans
                     yield m[0], m[1], simplify_message(m[2]), m[3]
                 else:
                     yield m
 
-    def extract_text(fileobj, keywords, comment_tags, options):
-        """Extract messages from Genshi or Jinja2 text templates.
 
-        This is only needed as an interim measure, as long as we have both.
-        """
-        for m in extract_html(fileobj, keywords, comment_tags, options,
-                              text=True):
-            yield m
+    extract_text = extract_html
 
 
     class generate_messages_js(Command):
@@ -415,7 +386,7 @@ try:
                     t = Translations(infile, self.domain)
                     catalog = t._catalog
 
-                with open(js_file, 'w') as outfile:
+                with open(js_file, 'w', encoding='utf-8') as outfile:
                     write_js(outfile, catalog, self.domain, locale)
 
 
@@ -448,7 +419,7 @@ try:
         def run(self):
             for filename in self._get_po_files():
                 distlog.info('checking catalog %s', filename)
-                with open(filename) as f:
+                with open(filename, 'rb') as f:
                     catalog = read_po(f, domain=self.domain)
                 for message in catalog:
                     for error in self._check_message(catalog, message):
@@ -476,11 +447,6 @@ try:
                 yield e
             for e in check_markup(catalog, message):
                 yield e
-            if genshi: # TODO (1.5.1) remove
-                try:
-                    check_genshi_markup(catalog, message)
-                except TranslationError as e:
-                    yield e
 
     def check_markup(catalog, message):
         """Verify markups in the translation."""
@@ -536,7 +502,7 @@ try:
         data = {'domain': domain, 'locale': locale}
 
         messages = {}
-        for msgid, msgstr in catalog.iteritems():
+        for msgid, msgstr in catalog.items():
             if isinstance(msgid, (list, tuple)):
                 messages.setdefault(msgid[0], {})
                 messages[msgid[0]][msgid[1]] = msgstr
@@ -555,11 +521,14 @@ try:
                         data['plural_expr'] = pluralexpr(val)
                         break
         data['messages'] = messages
+        data = to_json(data)
+        if isinstance(data, bytes):
+            data = str(data, 'utf-8')
 
         fileobj.write('// Generated messages javascript file '
                       'from compiled MO file\n')
         fileobj.write('babel.Translations.load(')
-        fileobj.write(to_json(data).encode('utf-8'))
+        fileobj.write(data)
         fileobj.write(').install();\n')
 
     def pluralexpr(forms):
@@ -649,46 +618,6 @@ try:
             'check_catalog_tracini': check_catalog,
         }
 
-    # TODO (1.5.1) remove
-    if genshi:
-        def check_genshi_markup(catalog, message):
-            """Verify the genshi markups in the translation."""
-            msgids = message.id
-            if not isinstance(msgids, (list, tuple)):
-                msgids = (msgids,)
-            msgstrs = message.string
-            if not isinstance(msgstrs, (list, tuple)):
-                msgstrs = (msgstrs,)
-
-            # check using genshi-markup
-            if not _GENSHI_MARKUP_SEARCH(msgids[0]):
-                return
-
-            for msgid, msgstr in izip(msgids, msgstrs):
-                if msgstr:
-                    _validate_genshi_markup(msgid, msgstr)
-
-
-        def _validate_genshi_markup(markup, alternative):
-            indices_markup = _parse_genshi_markup(markup)
-            indices_alternative = _parse_genshi_markup(alternative)
-            indices = indices_markup - indices_alternative
-            if indices:
-                raise TranslationError(
-                    'genshi markups are unbalanced %s' %
-                    ' '.join('[%d:]' % idx for idx in indices))
-
-
-        def _parse_genshi_markup(message):
-            from genshi.filters.i18n import parse_msg
-            try:
-                return {idx for idx, text in parse_msg(message) if idx > 0}
-            except Exception as e:
-                raise TranslationError('cannot parse message (%s: %s)' % \
-                                       (e.__class__.__name__, unicode(e)))
-
-
-
 except ImportError:
     def get_l10n_cmdclass():
         return
