@@ -39,8 +39,8 @@ In some cases, only the `url` pointing to the file's content is
 actually needed, that's why we avoid to read the file's content when
 it's not needed.
 
-The actual `content` to be converted might be a `unicode` object, but
-it can also be the raw byte string (`str`) object, or simply an object
+The actual `content` to be converted might be a `str` object, but
+it can also be the raw byte string (`bytes`) object, or simply an object
 that can be `read()`.
 
 .. note:: (for plugin developers)
@@ -353,8 +353,7 @@ for x in TEXT_X_TYPES.split():
 MIME_MAP = {}
 for t, exts in KNOWN_MIME_TYPES.items():
     MIME_MAP[t] = t
-    for e in exts:
-        MIME_MAP[e] = t
+    MIME_MAP.update((e, t) for e in exts)
 
 # Simple builtin autodetection from the content using a regexp
 MODE_RE = re.compile(r"""
@@ -362,7 +361,7 @@ MODE_RE = re.compile(r"""
     | \#!(?:[/\w.-_]+/)?(\w+)               # 2. look for regular shebang
     | -\*-\s*(?:mode:\s*)?([\w+-]+)\s*-\*-  # 3. look for Emacs' -*- mode -*-
     | vim:.*?(?:syntax|filetype|ft)=(\w+)   # 4. look for VIM's syntax=<n>
-    """, re.VERBOSE)
+    """.encode('utf-8'), re.VERBOSE)
 
 
 def get_mimetype(filename, content=None, mime_map=MIME_MAP,
@@ -372,10 +371,10 @@ def get_mimetype(filename, content=None, mime_map=MIME_MAP,
     `filename` is either a filename (the lookup will then use the suffix)
     or some arbitrary keyword.
 
-    `content` is either a `str` or an `unicode` string.
+    `content` is either a `bytes` or a `str` string.
     """
     # 0) mimetype from filename pattern (most specific)
-    for mimetype, regexp in mime_map_patterns.iteritems():
+    for mimetype, regexp in mime_map_patterns.items():
         if regexp.match(filename):
             return mimetype
     suffix = filename.split('.')[-1]
@@ -390,10 +389,15 @@ def get_mimetype(filename, content=None, mime_map=MIME_MAP,
         except Exception:
             pass
         if not mimetype and content:
-            match = re.search(MODE_RE, content[:1000] + content[-1000:])
+            chunk = (content[:1000], content[-1000:])
+            if isinstance(content, str):
+                chunk = map(lambda v: v.encode('utf-8'), chunk)
+            chunk = b'\n\n'.join(chunk)
+            match = MODE_RE.search(chunk)
             if match:
-                mode = match.group(1) or match.group(2) or match.group(4) or \
-                    match.group(3).lower()
+                mode = str(match.group(1) or match.group(2) or
+                           match.group(4) or match.group(3).lower(),
+                           'iso-8859-1')
                 if mode in mime_map:
                     # 3) mimetype from the content, using the `MODE_RE`
                     return mime_map[mode]
@@ -412,46 +416,51 @@ def ct_mimetype(content_type):
 def is_binary(data):
     """Detect binary content by checking the first thousand bytes for zeroes.
 
-    Operate on either `str` or `unicode` strings.
+    Operate on either `bytes` or `str` strings.
     """
-    if isinstance(data, str) and detect_unicode(data):
-        return False
-    return '\0' in data[:1000]
+    if isinstance(data, bytes):
+        if detect_unicode(data):
+            return False
+        return data.find(0, 0, 1000) != -1
+    if isinstance(data, str):
+        return data.find('\0', 0, 1000) != -1
+    return False
 
 
 def detect_unicode(data):
     """Detect different unicode charsets by looking for BOMs (Byte Order Mark).
 
-    Operate obviously only on `str` objects.
+    Operate obviously only on `bytes` objects.
     """
-    if data.startswith('\xff\xfe'):
+    if data.startswith(b'\xff\xfe'):
         return 'utf-16-le'
-    elif data.startswith('\xfe\xff'):
+    elif data.startswith(b'\xfe\xff'):
         return 'utf-16-be'
-    elif data.startswith('\xef\xbb\xbf'):
+    elif data.startswith(b'\xef\xbb\xbf'):
         return 'utf-8'
     else:
         return None
 
 
 def content_to_unicode(env, content, mimetype):
-    """Retrieve an `unicode` object from a `content` to be previewed.
+    """Retrieve an `str` object from a `content` to be previewed.
 
     In case the raw content had an unicode BOM, we remove it.
 
     >>> from trac.test import EnvironmentStub
     >>> env = EnvironmentStub()
-    >>> content_to_unicode(env, u"\ufeffNo BOM! h\u00e9 !", '')
-    u'No BOM! h\\xe9 !'
-    >>> content_to_unicode(env, "\xef\xbb\xbfNo BOM! h\xc3\xa9 !", '')
-    u'No BOM! h\\xe9 !'
+    >>> content_to_unicode(env, "\ufeffNo BOM! h\u00e9 !", '')
+    'No BOM! h\\xe9 !'
+    >>> content_to_unicode(env, bytes([0xEF, 0xBB, 0xBF]) + b'No BOM! h' +
+    ...                         bytes([0xC3, 0xA9]) + b' !', '')
+    'No BOM! h\\xe9 !'
 
     """
     mimeview = Mimeview(env)
     if hasattr(content, 'read'):
         content = content.read(mimeview.max_preview_size)
     u = mimeview.to_unicode(content, mimetype)
-    if u and u[0] == u'\ufeff':
+    if u and u[0] == '\ufeff':
         u = u[1:]
     return u
 
@@ -512,7 +521,7 @@ class IHTMLPreviewRenderer(Interface):
         `RenderingContext`.
 
         The `content` might be:
-         * a `str` object
+         * a `bytes` object
          * an `unicode` string
          * any object with a `read` method, returning one of the above
 
@@ -583,8 +592,8 @@ class IContentConverter(Interface):
         represented by key. Returns a tuple in the form (content,
         output_mime_type) or None if conversion is not possible.
 
-        content must be a `str` instance or an iterable instance which
-        iterates `str` instances."""
+        content must be a `bytes` instance or an iterable instance which
+        iterates `bytes` instances."""
 
 
 class Content(object):
@@ -703,15 +712,27 @@ class Mimeview(Component):
             output = conversion.converter.convert_content(req, mimetype,
                                                           content,
                                                           conversion.key)
-            if output:
-                content, content_type = output
-                if iterable:
-                    if isinstance(content, basestring):
-                        content = (content,)
-                else:
-                    if not isinstance(content, basestring):
-                        content = ''.join(content)
-                return content, content_type, conversion.extension
+            if not output:
+                continue
+            content, content_type = output
+            is_iterable = not isinstance(content, (str, bytes))
+            if iterable:
+                if not is_iterable:
+                    content = (content,)
+            else:
+                if is_iterable:
+                    content = iter(content)
+                    try:
+                        first = next(content)
+                    except StopIteration:
+                        content = b''  # the iterable is empty
+                    else:
+                        join = ''.join if isinstance(first, str) else b''.join
+                        def g():
+                            yield first
+                            yield from content
+                        content = join(g())
+            return content, content_type, conversion.extension
         raise TracError(
             _("No available MIME conversions from %(old)s to %(new)s",
               old=mimetype, new=key))
@@ -806,8 +827,8 @@ class Mimeview(Component):
                 if not (force_source or getattr(renderer, 'returns_source',
                                                 False)):
                     # Direct rendering of content
-                    if isinstance(result, basestring):
-                        return Markup(to_unicode(result))
+                    if isinstance(result, str):
+                        return Markup(result)
                     elif isinstance(result, Fragment):
                         return result.__html__()
                     else:
@@ -836,7 +857,7 @@ class Mimeview(Component):
                 annotators[atype] = annotator
         annotations = [a for a in annotations if a in annotators]
 
-        if isinstance(lines, unicode):
+        if isinstance(lines, str):
             lines = lines.splitlines(True)
         # elif isinstance(lines, list):
         #    pass # assume these are lines already
@@ -858,7 +879,7 @@ class Mimeview(Component):
             return tag.tr(
                 [tag.th(labels[a], class_=a, title=titles[a])
                  for a in annotations] +
-                [tag.th(u'\xa0', class_='content')]
+                [tag.th('\xa0', class_='content')]
             )
 
         def _body_rows():
@@ -881,7 +902,7 @@ class Mimeview(Component):
     def get_charset(self, content='', mimetype=None):
         """Infer the character encoding from the `content` or the `mimetype`.
 
-        `content` is either a `str` or an `unicode` object.
+        `content` is either a `bytes` or a `str` object.
 
         The charset will be determined using this order:
          * from the charset information present in the `mimetype` argument
@@ -892,7 +913,7 @@ class Mimeview(Component):
             ctpos = mimetype.find('charset=')
             if ctpos >= 0:
                 return mimetype[ctpos + 8:].strip()
-        if isinstance(content, str):
+        if isinstance(content, bytes):
             utf = detect_unicode(content)
             if utf is not None:
                 return utf
@@ -921,7 +942,7 @@ class Mimeview(Component):
     def get_mimetype(self, filename, content=None):
         """Infer the MIME type from the `filename` or the `content`.
 
-        `content` is either a `str` or an `unicode` object.
+        `content` is either a `bytes` or a `str` object.
 
         Return the detected MIME type, augmented by the
         charset information (i.e. "<mimetype>; charset=..."),
@@ -966,7 +987,7 @@ class Mimeview(Component):
         return False
 
     def to_unicode(self, content, mimetype=None, charset=None):
-        """Convert `content` (an encoded `str` object) to an `unicode` object.
+        """Convert `content` (an encoded `bytes` object) to a `str` object.
 
         This calls `trac.util.to_unicode` with the `charset` provided,
         or the one obtained by `Mimeview.get_charset()`.
@@ -1021,13 +1042,13 @@ class Mimeview(Component):
         if iterable:
             def encoder(content):
                 for chunk in content:
-                    if isinstance(chunk, unicode):
+                    if isinstance(chunk, str):
                         chunk = chunk.encode('utf-8')
                     yield chunk
             content = encoder(content)
             length = None
         else:
-            if isinstance(content, unicode):
+            if isinstance(content, str):
                 content = content.encode('utf-8')
             length = len(content)
         req.send_response(200)
