@@ -20,10 +20,11 @@ import hashlib
 import os
 import re
 import smtplib
+from email import policy
 from email.charset import BASE64, QP, SHORTEST, Charset
 from email.header import Header
-from email.mime.multipart import MIMEMultipart
-from email.mime.text import MIMEText
+from email.headerregistry import Address
+from email.message import EmailMessage
 from email.utils import formatdate, parseaddr, getaddresses
 from subprocess import Popen, PIPE
 
@@ -64,6 +65,13 @@ _mime_encoding_re = re.compile(r'=\?[^?]+\?[bq]\?[^?]+\?=', re.IGNORECASE)
 
 local_hostname = None
 
+# When default policy is used if [notification] mime_encoding is 'none'
+# and a line is exceeded max_line_length (78 bytes by default), 'base64'
+# or 'quoted-printable' is used for Content-Transfer-Encoding header.
+# Using the custom policy avoids the behavior.
+_policy_default = policy.SMTP  # newline is CRLF
+_policy_8bit = _policy_default.clone(max_line_length=998)
+
 
 def create_charset(mime_encoding):
     """Create an appropriate email charset for the given encoding.
@@ -112,7 +120,7 @@ def create_header(key, value, charset):
     email = None
     if isinstance(value, (tuple, list)):
         value, email = value
-    if not isinstance(value, basestring):
+    if not isinstance(value, str):
         value = to_unicode(value)
     if not value:
         return email
@@ -137,30 +145,85 @@ def create_header(key, value, charset):
     return header
 
 
-def set_header(message, key, value, charset):
-    """Create and add or replace a header in a `MIMEMultipart`.
+def create_address_header(addresses):
+    """Create address header instance to pass to `set_header`.
 
-    The `key` is always a string and will be converted to the
-    appropriate `charset`. The `value` can either be a string or a
-    two-element tuple where the first item is the name and the
-    second item is the email address.
+    The `addresses` is a list or an iterable of addresses. The item can
+    either be `str`, a `(name, address)` tuple or a `(None, address)`.
+    """
+    if isinstance(addresses, str):
+        addresses = [(None, addresses)]
+    l = []
+    for item in addresses:
+        if isinstance(item, Address):
+            l.append(item)
+            continue
+        if isinstance(item, str):
+            name = None
+            addr = item
+        elif isinstance(item, (list, tuple)):
+            name, addr = item
+        else:
+            raise ValueError('Unrecognized item %r' % item)
+        if '@' in addr:
+            username, domain = addr.rsplit('@', 1)
+        else:
+            username = addr
+            domain = ''
+        l.append(Address(_replace_encoded_words(name or ''), username, domain))
+    return l
 
-    The `charset` should be created using `create_charset()`
+
+def set_header(message, key, value=None, charset=None, addresses=None):
+    """Create and add or replace a header in a `EmailMessage`.
+
+    The `key` is always a string. The `value` can either be `None`, a
+    string or a two-element tuple where the first item is the name and
+    the second item is the email address.
+
+    The `addresses` can either be a list or an iterable of a two-element
+    tuple. When the `addresses` is given, the `value` will be ignored.
+
+    The `charset` is no longer used.
 
     Example::
 
-        set_header(my_message, 'From', ('Trac', 'noreply@ourcompany.com'),
-                   my_charset)
+        set_header(my_message, 'From', ('Trac', 'noreply@ourcompany.com'))
+
+        set_header(my_message, 'To',
+                   addresses=[('Foo', 'foo@example.org'),
+                              ('Bar', 'bar@example.org')])
     """
-    header = create_header(key, value, charset)
+    if addresses is not None:
+        header = create_address_header(addresses)
+    elif isinstance(value, (list, tuple)):  # a pair of name and address
+        header = create_address_header([value])
+    elif isinstance(value, Address):
+        header = value
+    elif value is None:
+        header = ''
+    else:
+        header = _replace_encoded_words(str(value))
     if key in message:
         message.replace_header(key, header)
     else:
         message[key] = header
 
 
+_encoded_words_re = re.compile(r'=\?')
+
+
+def _replace_encoded_words(text):
+    """Replace '=?' with '=\u200b?' to avoid decoding encoded-words by
+    `EmailMessage`.
+    """
+    if text:
+        text = _encoded_words_re.sub('=\u200b?', text)
+    return text
+
+
 def create_mime_multipart(subtype):
-    """Create an email `MIMEMultipart`.
+    """Create a multipart email message.
 
     The `subtype` is a string that describes the type of multipart
     message you are defining. You should pick one that is defined
@@ -175,32 +238,36 @@ def create_mime_multipart(subtype):
       and the client can choose which to display based on capabilities
       and user preferences, such as a text/html with an alternative
       text/plain.
-
-    The `MIMEMultipart` is defined in the `email.mime.multipart` module in the
-    Python standard library.
     """
-    msg = MIMEMultipart(subtype)
-    del msg['Content-Transfer-Encoding']
+    msg = EmailMessage()
+    if subtype == 'related':
+        msg.make_related()
+    elif subtype == 'alternative':
+        msg.make_alternative()
+    elif subtype == 'mixed':
+        msg.make_mixed()
+    else:
+        raise ValueError("subtype must be one of ('related', 'multipart', "
+                         "'mixed'), not %r" % subtype)
+    msg['MIME-Version'] = '1.0'
     return msg
 
 
 def create_mime_text(body, format, charset):
-    """Create a `MIMEText` that can be added to an email message.
+    """Create a `EmailMessage` that can be added to an email message.
 
     :param body: a string with the body of the message.
-    :param format: each text has a MIMEType, like `text/plain`. The
+    :param format: each text has a EmailMessage, like `text/plain`. The
         supertype is always `text`, so in the `format` parameter you
         pass the subtype, like `plain` or `html`.
     :param charset: should be created using `create_charset()`.
     """
-    if isinstance(body, unicode):
-        body = body.encode('utf-8')
-    msg = MIMEText(body, format)
-    # Message class computes the wrong type from MIMEText constructor,
-    # which does not take a Charset object as initializer. Reset the
-    # encoding type to force a new, valid evaluation
-    del msg['Content-Transfer-Encoding']
-    msg.set_charset(charset)
+    if isinstance(body, bytes):
+        body = str(body, 'utf-8')
+    cte = {BASE64: 'base64', QP: 'quoted-printable'}.get(charset.body_encoding)
+    msg = EmailMessage(_policy_8bit if cte is None else _policy_default)
+    msg['MIME-Version'] = '1.0'
+    msg.set_content(body, subtype=format, cte=cte)
     return msg
 
 
@@ -219,10 +286,12 @@ def create_message_id(env, targetid, from_email, time, more=None):
     :param more: a string that contains additional information that
         makes this message unique
     """
-    items = [env.project_url.encode('utf-8'), targetid, to_utimestamp(time)]
+    items = [env.project_url, targetid, to_utimestamp(time)]
     if more is not None:
         items.append(more.encode('ascii', 'ignore'))
-    source = '.'.join(str(item) for item in items)
+    source = b'.'.join(item if isinstance(item, bytes) else
+                       str(item).encode('utf-8')
+                       for item in items)
     hash_type = NotificationSystem(env).message_id_hash
     try:
         h = hashlib.new(hash_type)
@@ -480,7 +549,7 @@ class EmailDistributor(Component):
 
         outputs = {}
         failed = []
-        for fmt, formatter in formats.iteritems():
+        for fmt, formatter in formats.items():
             if fmt not in addresses and fmt != 'text/plain':
                 continue
             try:
@@ -499,7 +568,7 @@ class EmailDistributor(Component):
                 addresses.setdefault('text/plain', set()) \
                          .update(addresses.pop(fmt, ()))
 
-        for fmt, addrs in addresses.iteritems():
+        for fmt, addrs in addresses.items():
             self.log.debug("%s is sending event as '%s' to: %s",
                            self.__class__.__name__, fmt, ', '.join(addrs))
             message = self._create_message(fmt, outputs)
@@ -520,9 +589,10 @@ class EmailDistributor(Component):
         maintype, subtype = format.split('/')
         preferred = create_mime_text(outputs[format], subtype, self._charset)
         if format != 'text/plain' and 'text/plain' in outputs:
+            text = create_mime_text(outputs['text/plain'], 'plain',
+                                    self._charset)
             alternative = create_mime_multipart('alternative')
-            alternative.attach(create_mime_text(outputs['text/plain'],
-                                                'plain', self._charset))
+            alternative.attach(text)
             alternative.attach(preferred)
             preferred = alternative
         message.attach(preferred)
@@ -565,17 +635,18 @@ class EmailDistributor(Component):
             headers['In-Reply-To'] = rootid
             headers['References'] = rootid
         headers['Date'] = formatdate()
-        headers['From'] = (smtp_from_name, smtp_from) \
-                          if smtp_from_name else smtp_from
         headers['To'] = 'undisclosed-recipients: ;'
+        for k, v in headers.items():
+            set_header(message, k, v)
+
+        set_header(message, 'From', (smtp_from_name, smtp_from)
+                                    if smtp_from_name else smtp_from)
         if cc_addrs:
-            headers['Cc'] = ', '.join(cc_addrs)
+            set_header(message, 'Cc', addresses=cc_addrs)
         if bcc_addrs:
-            headers['Bcc'] = ', '.join(bcc_addrs)
-        headers['Reply-To'] = smtp_replyto
+            set_header(message, 'Bcc', addresses=bcc_addrs)
+        set_header(message, 'Reply-To', addresses=[smtp_replyto])
 
-        for k, v in headers.iteritems():
-            set_header(message, k, v, self._charset)
         for decorator in self.decorators:
             decorator.decorate_message(event, message, self._charset)
 
@@ -586,7 +657,7 @@ class EmailDistributor(Component):
             to_addrs.update(addr for name, addr in getaddresses(values)
                                  if addr)
         del message['Bcc']
-        notify_sys.send_email(from_addr, list(to_addrs), message.as_string())
+        notify_sys.send_email(from_addr, list(to_addrs), message.as_bytes())
 
 
 class SmtpEmailSender(Component):
@@ -751,4 +822,4 @@ class FromAuthorEmailDecorator(Component):
     def decorate_message(self, event, message, charset):
         from_ = get_from_author(self.env, event)
         if from_:
-            set_header(message, 'From', from_, charset)
+            set_header(message, 'From', addresses=[from_])
