@@ -16,8 +16,6 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import print_function
-
 import csv
 import os
 from itertools import groupby
@@ -27,7 +25,7 @@ from trac.cache import cached
 from trac.config import ExtensionOption, OrderedExtensionsOption
 from trac.core import *
 from trac.resource import Resource, get_resource_name
-from trac.util import file_or_std
+from trac.util import file_or_std, lazy
 from trac.util.datefmt import time_now
 from trac.util.text import path_to_unicode, print_table, printout, \
                            stream_encoding, to_unicode, wrap
@@ -412,6 +410,10 @@ class PermissionSystem(Component):
                     actions.setdefault(action, [])
         return actions
 
+    @lazy
+    def actions(self):
+        return self.get_actions()
+
     def get_actions(self, skip=None):
         """Get a list of all actions defined by permission requestors."""
         actions = set()
@@ -518,7 +520,7 @@ class PermissionSystem(Component):
             return permissions
 
         parent_map = {}
-        for parent, children in self.get_actions_dict().iteritems():
+        for parent, children in self.get_actions_dict().items():
             for child in children:
                 parent_map.setdefault(child, set()).add(parent)
 
@@ -811,15 +813,14 @@ class PermissionAdmin(Component):
 
     def _do_export(self, filename=None):
         try:
-            with file_or_std(filename, 'wb') as f:
-                encoding = stream_encoding(f)
+            with file_or_std(filename, 'w') as f:
                 linesep = os.linesep if filename else '\n'
                 writer = csv.writer(f, lineterminator=linesep)
                 users = self.get_user_list()
                 for user in sorted(users):
                     actions = sorted(self.get_user_perms(user))
-                    writer.writerow([s.encode(encoding, 'replace')
-                                     for s in [user] + actions])
+                    writer.writerow([s for s in [user] + actions])
+                f.flush()
         except IOError as e:
             raise AdminCommandError(
                 _("Cannot export to %(filename)s: %(error)s",
@@ -830,8 +831,7 @@ class PermissionAdmin(Component):
         permsys = PermissionSystem(self.env)
         linesep = os.linesep if filename else '\n'
         try:
-            with file_or_std(filename, 'rb') as f:
-                encoding = stream_encoding(f)
+            with file_or_std(filename, 'r') as f:
                 reader = csv.reader(f, lineterminator=linesep)
                 for row in reader:
                     if len(row) < 2:
@@ -839,9 +839,8 @@ class PermissionAdmin(Component):
                             _("Invalid row %(line)d. Expected <user>, "
                               "<action>, [action], [...]",
                               line=reader.line_num))
-                    user = to_unicode(row[0], encoding)
-                    actions = [to_unicode(action, encoding)
-                               for action in row[1:]]
+                    user = row[0]
+                    actions = row[1:]
                     if user.isupper():
                         raise AdminCommandError(
                             _("Invalid user %(user)s on line %(line)d: All "
