@@ -30,12 +30,4 @@ class IPreferencePanelProvider(Interface):
         This function should return a tuple of the form `(template,
         data)`, where `template` is the name of the template to use
         and `data` is the data used when rendering the template.
-
-        .. note::
-
-           When a plugin wants to use a legacy Genshi template instead
-           of a Jinja2 template, it needs to return instead a *triple*
-           of the form ``(template, data, None)``, similar to what
-           `.IRequestHandler.process_request` does.
-
         """
