@@ -84,7 +84,7 @@ class SearchModule(Component):
                is not None
 
     def process_request(self, req):
-        req.perm.assert_permission('SEARCH_VIEW')
+        req.perm.require('SEARCH_VIEW')
 
         if req.path_info == '/search/opensearch':
             return ('opensearch.xml', {},
@@ -192,7 +192,7 @@ class SearchModule(Component):
                         'description': description}
             else:
                 help_url = req.href.wiki('TracSearch') + '#Quicksearches'
-                search_url = req.href.search(q=kwd, noquickjump=1)
+                search_url = req.href.search([('q', kwd), ('noquickjump', 1)])
                 # FIXME: use tag_
                 add_notice(req, Markup(_(
                     'You arrived here through the <a href="%(help_url)s">'
