@@ -303,8 +303,8 @@ class TicketSystem(Component):
                     actions[action] = max(actions[action], weight)
                 else:
                     actions[action] = weight
-        all_weighted_actions = [(weight, action) for action, weight in
-                                actions.items()]
+        all_weighted_actions = [(weight, action) for action, weight
+                                                 in actions.items()]
         return [x[1] for x in sorted(all_weighted_actions, reverse=True)]
 
     def get_all_status(self):
@@ -433,6 +433,11 @@ class TicketSystem(Component):
                          name.replace("_", " ").strip().capitalize(),
                 'value': config.get(name + '.value', '')
             }
+
+            def _get_ticketlink_query():
+                field['ticketlink_query'] = \
+                        config.get(name + '.ticketlink_query', None)
+
             if field['type'] == 'select' or field['type'] == 'radio':
                 field['options'] = config.getlist(name + '.options', sep='|')
                 if not field['options']:
@@ -442,11 +447,15 @@ class TicketSystem(Component):
                     field['optional'] = True
                     if '' in field['options']:
                         field['options'].remove('')
+                _get_ticketlink_query()
             elif field['type'] == 'checkbox':
                 field['value'] = '1' if as_bool(field['value']) else '0'
+                _get_ticketlink_query()
             elif field['type'] == 'text':
                 field['format'] = config.get(name + '.format', 'plain')
                 field['max_size'] = config.getint(name + '.max_size', 0)
+                if field['format'] in ('reference', 'list'):
+                    _get_ticketlink_query()
             elif field['type'] == 'textarea':
                 field['format'] = config.get(name + '.format', 'plain')
                 field['max_size'] = config.getint(name + '.max_size', 0)
@@ -632,8 +641,8 @@ class TicketSystem(Component):
                 ranges = str(r)
                 if params:
                     params = '&' + params[1:]
-                label_wrap = label.replace(',', u',\u200b')
-                ranges_wrap = ranges.replace(',', u', ')
+                label_wrap = label.replace(',', ',\u200b')
+                ranges_wrap = ranges.replace(',', ', ')
                 return tag.a(label_wrap,
                              title=_("Tickets %(ranges)s", ranges=ranges_wrap),
                              href=formatter.href.query(id=ranges) + params)
