@@ -17,7 +17,7 @@
 # Author: Eli Carter
 
 import io
-from ConfigParser import ParsingError, RawConfigParser
+from configparser import ParsingError, RawConfigParser
 from collections import defaultdict
 from functools import partial
 from pkg_resources import resource_filename
@@ -90,6 +90,8 @@ def parse_workflow_config(rawactions):
                 attributes['label'] = action.replace("_", " ").strip()
         for key, val in required_attrs.items():
             attributes.setdefault(key, val)
+        for val in ('<none>', '< none >'):
+            sub_val(attributes['oldstates'], val, None)
 
     return actions
 
@@ -177,9 +179,12 @@ class ConfigurableTicketWorkflow(Component):
         for action_name, action_info in self.actions.items():
             operations = action_info['operations']
             newstate = action_info['newstate']
+            if 'leave_status' not in operations and newstate == '*':
+                # Some other controller handles this operation
+                continue
             # Exclude action that is effectively a No-op.
             if len(operations) == 1 and \
-                    operations[0] == 'set_owner_to_self' and \
+                    'set_owner_to_self' in operations and \
                     ticket_owner == author and ticket_status == newstate:
                 continue
             oldstates = action_info['oldstates']
@@ -364,8 +369,6 @@ class ConfigurableTicketWorkflow(Component):
                                   current_owner=formatted_current_owner)
                              if ticket_owner else
                              _("The ticket will remain with no owner"))
-        elif next_status == '*':
-            label = None  # Control won't be created
         elif ticket['status'] is None:  # New ticket
             hints.append(tag_("The status will be '%(name)s'",
                               name=next_status))
@@ -455,7 +458,7 @@ class ConfigurableTicketWorkflow(Component):
         actions = parse_workflow_config(self.ticket_workflow_section.options())
 
         has_new_state = any('new' in [a['newstate']] + a['oldstates']
-                            for a in actions.itervalues())
+                            for a in actions.values())
         if has_new_state:
             # Special action that gets enabled if the current status no
             # longer exists, as no other action can then change its state.
@@ -471,9 +474,7 @@ class ConfigurableTicketWorkflow(Component):
             for key, val in reset.items():
                 actions['_reset'].setdefault(key, val)
 
-        for name, info in actions.iteritems():
-            for val in ('<none>', '< none >'):
-                sub_val(actions[name]['oldstates'], val, None)
+        for name, info in actions.items():
             if not info['newstate']:
                 self.log.warning("Ticket workflow action '%s' doesn't define "
                                  "any transitions", name)
@@ -674,9 +675,9 @@ class WorkflowMacro(WikiMacroBase):
             raw_actions = list(parser.items('ticket-workflow'))
         actions = parse_workflow_config(raw_actions)
         states = list(
-            {state for action in actions.itervalues()
+            {state for action in actions.values()
                    for state in action['oldstates']} |
-            {action['newstate'] for action in actions.itervalues()})
+            {action['newstate'] for action in actions.values()})
         action_labels = [attrs['label'] for attrs in actions.values()]
         action_names = list(actions)
         edges = []
@@ -694,7 +695,6 @@ class WorkflowMacro(WikiMacroBase):
                  'width': width, 'height': height}
         graph_id = '%012x' % id(graph)
         req = formatter.req
-        add_script(req, 'common/js/excanvas.js', ie_if='IE')
         add_script(req, 'common/js/workflow_graph.js')
         add_script_data(req, {'graph_%s' % graph_id: graph})
         return tag(
