@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2017-2019 Edgewall Software
+# Copyright (C) 2017-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -21,6 +21,8 @@ def insert_ticket(env, **props):
     """
     when = props.pop('when', None)
     ticket = Ticket(env)
+    ticket['status'] = 'new'
+    ticket['summary'] = 'the summary'
     for k, v in props.items():
         ticket[k] = v
     ticket.insert(when)
