@@ -26,7 +26,7 @@ from trac.config import BoolOption, Option
 from trac.core import *
 from trac.mimeview.api import Mimeview, IContentConverter
 from trac.notification.api import NotificationSystem
-from trac.perm import IPermissionPolicy
+from trac.perm import IPermissionPolicy, PermissionSystem
 from trac.resource import (
     Resource, ResourceNotFound, get_resource_url, render_resource_link,
     get_resource_shortname
@@ -60,14 +60,6 @@ from trac.web.chrome import (
 from trac.wiki.formatter import format_to, format_to_html
 
 
-class InvalidTicket(TracError):
-    """Exception raised when a ticket fails validation.
-
-    :since 1.3.2: deprecated and will be removed in 1.5.1
-    """
-    title = N_("Invalid Ticket")
-
-
 class TicketModule(Component):
 
     implements(IContentConverter, INavigationContributor, IRequestHandler,
@@ -113,20 +105,6 @@ class TicketModule(Component):
     def __init__(self):
         self._warn_for_moved_attr = set()
 
-    def __getattr__(self, name):
-        """Delegate access to Options which were moved to TicketSystem.
-
-        .. todo:: remove in 1.5.1
-        """
-        if name in ('max_comment_size', 'max_description_size',
-                    'max_summary_size'):
-            if name not in self._warn_for_moved_attr:
-                self.log.warning("%s option should be accessed via "
-                                 "TicketSystem component", name)
-                self._warn_for_moved_attr.add(name)
-            return getattr(TicketSystem(self.env), name)
-        raise AttributeError("TicketModule has no attribute '%s'" % name)
-
     @lazy
     def must_preserve_newlines(self):
         preserve_newlines = self.preserve_newlines
@@ -430,7 +408,7 @@ class TicketModule(Component):
         if field == 'url':
             return context.href.query(id=','.join(str(t) for t in tickets))
         elif field == 'title':
-            ticketids = u',\u200b'.join(str(t) for t in tickets)
+            ticketids = ',\u200b'.join(str(t) for t in tickets)
             title = _("Tickets %(ticketids)s", ticketids=ticketids)
             return tag_("Tickets %(ticketlist)s batch updated",
                         ticketlist=tag.em('#', ticketids, title=title))
@@ -456,8 +434,6 @@ class TicketModule(Component):
                                                            action):
                 label, widget, hint = controller.render_ticket_action_control(
                     req, ticket, action)
-                if label is None:
-                    continue
                 labels.append(label)
                 widgets.append(widget)
                 hints.append(hint)
@@ -557,9 +533,6 @@ class TicketModule(Component):
         fields = self._prepare_fields(req, ticket)
 
         data['fields'] = fields
-        # fields_map is deprecated and removed in 1.5.1
-        data['fields_map'] = {field['name']: i
-                              for i, field in enumerate(fields)}
 
         if req.is_xhr:
             data['preview_mode'] = True
@@ -862,7 +835,7 @@ class TicketModule(Component):
         for each in model.Ticket.protected_fields:
             fields.pop(each, None)
             fields.pop('checkbox_' + each, None)    # See Ticket.populate()
-        for field, value in fields.iteritems():
+        for field, value in fields.items():
             if field in ticket.time_fields:
                 try:
                     fields[field] = user_time(req, parse_date, value) \
@@ -968,9 +941,9 @@ class TicketModule(Component):
 
         # assume a linear sequence of change numbers, starting at 1, with gaps
         def replay_changes(values, old_values, from_version, to_version):
-            for version in xrange(from_version, to_version+1):
+            for version in range(from_version, to_version+1):
                 if version in changes:
-                    for k, v in changes[version]['fields'].iteritems():
+                    for k, v in changes[version]['fields'].items():
                         values[k] = v['new']
                         if old_values is not None and k not in old_values:
                             old_values[k] = v['old']
@@ -1002,7 +975,7 @@ class TicketModule(Component):
 
         # -- prop changes
         props = []
-        for k, v in new_ticket.iteritems():
+        for k, v in new_ticket.items():
             if k not in text_fields:
                 old, new = old_ticket[k], new_ticket[k]
                 if old != new:
@@ -1214,11 +1187,13 @@ class TicketModule(Component):
                   if f['name'] not in ('time', 'changetime')]
         content = io.BytesIO()
         content.write(b'\xef\xbb\xbf')   # BOM
-        writer = csv.writer(content, delimiter=sep, quoting=csv.QUOTE_MINIMAL)
-        writer.writerow(['id'] + [unicode(f['name']) for f in fields])
+        writer = csv.writer(io.TextIOWrapper(content, encoding='utf-8',
+                                             newline='\n', write_through=True),
+                            delimiter=sep, quoting=csv.QUOTE_MINIMAL)
+        writer.writerow(['id'] + [f['name'] for f in fields])
 
         context = web_context(req, ticket.resource)
-        cols = [unicode(ticket.id)]
+        cols = [str(ticket.id)]
         for f in fields:
             name = f['name']
             value = ticket[name] or ''
@@ -1228,7 +1203,7 @@ class TicketModule(Component):
                 format = ticket.fields.by_name(name).get('format')
                 value = user_time(req, format_date_or_datetime, format,
                                   value) if value else ''
-            cols.append(value.encode('utf-8'))
+            cols.append(value)
         writer.writerow(cols)
         return content.getvalue(), '%s;charset=utf-8' % mimetype
 
@@ -1242,7 +1217,7 @@ class TicketModule(Component):
             # wikify comment
             if 'comment' in change:
                 change_summary['added'] = ['comment']
-            for field, values in change['fields'].iteritems():
+            for field, values in change['fields'].items():
                 if field == 'description':
                     change_summary.setdefault('changed', []).append(field)
                 else:
@@ -1444,7 +1419,7 @@ class TicketModule(Component):
             field_changes[field] = {'old': old, 'new': new, 'by': author,
                                     'label': field_labels.get(field, field)}
         # Start with user changes
-        for field, value in ticket._old.iteritems():
+        for field, value in ticket._old.items():
             store_change(field, value or '', ticket[field], 'user')
 
         # Apply controller changes corresponding to the selected action
@@ -1470,7 +1445,7 @@ class TicketModule(Component):
                 store_change(key, old, new, cname)
 
         # Detect non-changes
-        for key, item in field_changes.items():
+        for key, item in list(field_changes.items()):
             if item['old'] == item['new']:
                 del field_changes[key]
         return field_changes, problems
@@ -1481,12 +1456,14 @@ class TicketModule(Component):
         for key in field_changes:
             ticket[key] = field_changes[key]['new']
 
-    def _query_link(self, req, name, value, text=None, class_=None):
+    def _query_link(self, req, name, value, text=None, class_=None, query=None):
         """Return a link to /query with the appropriate name and value"""
         from trac.ticket.query import QueryModule
         if not self.env.is_component_enabled(QueryModule):
             return text or value
-        args = arg_list_to_args(parse_arg_list(self.ticketlink_query))
+        if query is None:
+            query = self.ticketlink_query
+        args = arg_list_to_args(parse_arg_list(query))
         args[name] = value
         if name == 'resolution':
             args['status'] = 'closed'
@@ -1494,13 +1471,15 @@ class TicketModule(Component):
             return tag.a(text or value, href=req.href.query(args),
                          class_=class_)
 
-    def _query_link_words(self, context, name, value):
+    def _query_link_words(self, context, name, value, query=None):
         """Splits a list of words and makes a query link to each separately"""
         from trac.ticket.query import QueryModule
-        if not (isinstance(value, basestring) and  # None or other non-splitable
+        if not (isinstance(value, str) and  # None or other non-splitable
                 self.env.is_component_enabled(QueryModule)):
             return value
-        args = arg_list_to_args(parse_arg_list(self.ticketlink_query))
+        if query is None:
+            query = self.ticketlink_query
+        args = arg_list_to_args(parse_arg_list(query))
         items = []
         for i, word in enumerate(re.split(r'([;,\s]+)', value)):
             if i % 2:
@@ -1531,8 +1510,10 @@ class TicketModule(Component):
             field.setdefault('editable', True)
 
             # enable a link to custom query for all choice fields
+            ticketlink_query = field.get('ticketlink_query')
             if type_ not in ['text', 'textarea', 'time']:
-                field['rendered'] = self._query_link(req, name, ticket[name])
+                field['rendered'] = self._query_link(req, name, ticket[name],
+                                                     query=ticketlink_query)
 
             # per field settings
             if name in ('summary', 'reporter', 'description', 'owner',
@@ -1607,15 +1588,19 @@ class TicketModule(Component):
             elif type_ == 'checkbox':
                 value = ticket[name]
                 if value in ('1', '0'):
-                    field['rendered'] = self._query_link(req, name, value,
-                                _("yes") if value == '1' else _("no"))
+                    text = _("yes") if value == '1' else _("no")
+                    field['rendered'] = \
+                            self._query_link(req, name, value, text,
+                                             query=ticketlink_query)
             elif type_ == 'text':
                 if field.get('format') == 'reference':
-                    field['rendered'] = self._query_link(req, name,
-                                                         ticket[name])
+                    field['rendered'] = \
+                            self._query_link(req, name, ticket[name],
+                                             query=ticketlink_query)
                 elif field.get('format') == 'list':
-                    field['rendered'] = self._query_link_words(context, name,
-                                                               ticket[name])
+                    field['rendered'] = \
+                            self._query_link_words(context, name, ticket[name],
+                                                   query=ticketlink_query)
             elif type_ == 'time':
                 value = ticket[name]
                 field['timevalue'] = value
@@ -1647,8 +1632,6 @@ class TicketModule(Component):
         # -- Ticket fields
 
         fields = self._prepare_fields(req, ticket, field_changes)
-        # fields_map is deprecated and removed in 1.5.1
-        fields_map = {field['name']: i for i, field in enumerate(fields)}
 
         # -- Ticket Change History
 
@@ -1678,7 +1661,7 @@ class TicketModule(Component):
                 if ticket.resource.version is not None and \
                         cnum > ticket.resource.version:
                     # Retrieve initial ticket values from later changes
-                    for k, v in change['fields'].iteritems():
+                    for k, v in change['fields'].items():
                         if k not in values:
                             values[k] = v['old']
                     skip = True
@@ -1692,7 +1675,7 @@ class TicketModule(Component):
                                        'comment:%s' % replyto)
                     if ticket.resource.version:
                         # Override ticket value by current changes
-                        for k, v in change['fields'].iteritems():
+                        for k, v in change['fields'].items():
                             values[k] = v['new']
                     if 'description' in change['fields']:
                         data['description_change'] = change
@@ -1743,9 +1726,7 @@ class TicketModule(Component):
                     class_=chrome.author_class(req, user))
         data.update({
             'context': context, 'conflicts': conflicts,
-            'fields': fields,
-            'fields_map': fields_map,  # deprecated and removed in 1.5.1
-            'changes': changes, 'replies': replies,
+            'fields': fields, 'changes': changes, 'replies': replies,
             'attachments': AttachmentModule(self.env).attachment_data(context),
             'action_controls': action_controls, 'action': selected_action,
             'change_preview': change_preview, 'closetime': closetime,
@@ -1771,7 +1752,7 @@ class TicketModule(Component):
                 yield group
 
     def _render_property_changes(self, req, ticket, fields, resource_new=None):
-        for field, changes in fields.iteritems():
+        for field, changes in fields.items():
             new, old = changes['new'], changes['old']
             changes['rendered'] = \
                 self._render_property_diff(req, ticket, field, old, new,
@@ -1796,14 +1777,14 @@ class TicketModule(Component):
             if added or remvd:
                 return tag(added, added and remvd and _("; "), remvd)
             else:
-                return tag(old, u" \u2192 ", new)
+                return tag(old, " \u2192 ", new)
 
         def render_default(old, new):
             if old and new:
-                rendered = tag(tag.span(old, class_='trac-field-old'), u' → ',
+                rendered = tag(tag.span(old, class_='trac-field-old'), ' → ',
                                tag.span(new, class_='trac-field-new'))
             elif not old and new:
-                rendered = tag(u'→ ', tag.span(new, class_='trac-field-new'))
+                rendered = tag('→ ', tag.span(new, class_='trac-field-new'))
             else:  # old and not new
                 rendered = tag.span(old, class_='trac-field-deleted')
             return rendered
@@ -1926,6 +1907,7 @@ class DefaultTicketPolicy(Component):
     def check_permission(self, action, username, resource, perm):
 
         if action == 'TICKET_CHG_MILESTONE' and \
+                action not in PermissionSystem(self.env).actions and \
                 self._is_valid_resource(resource, 'milestone') and \
                 'MILESTONE_VIEW' in perm:
             return True
