@@ -15,8 +15,8 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 from abc import ABCMeta
-from BaseHTTPServer import BaseHTTPRequestHandler
-from Cookie import CookieError, BaseCookie, SimpleCookie
+from http.cookies import CookieError, BaseCookie, SimpleCookie
+from http.server import BaseHTTPRequestHandler
 import cgi
 from datetime import datetime
 import hashlib
@@ -25,8 +25,8 @@ import mimetypes
 import os
 import re
 import sys
-import urllib
-import urlparse
+import tempfile
+import urllib.parse
 
 from trac.core import Interface, TracBaseError, TracError
 from trac.util import as_bool, as_int, get_last_traceback, lazy, \
@@ -114,10 +114,11 @@ class IRequestHandler(Interface):
            API.  Note that the old API (`(template, data,
            content_type)` where `content_type` is a string or `None`)
            is still supported. When used, this means that `template`
-           is a legacy Genshi template. This support for the old API
-           will be removed in Trac 1.5.1, in which `metadata` will
-           always be a `dict` or `None` when specified.
+           is a legacy Genshi template.
 
+        :Since 1.5.1: Genshi templates are no longer supported. The
+            third argument, `metadata`, will always be a `dict` or `None`
+            when specified.
         """
 
 
@@ -140,28 +141,18 @@ class IRequestFilter(Interface):
         Always returns the request handler, even if unchanged.
         """
 
-    def post_process_request(req, template, data, metadata=None, method=None):
+    def post_process_request(req, template, data, metadata):
         """Do any post-processing the request might need
 
         This typically means adding values to the template `data`
         dictionary, or changing the Jinja2 template.
 
-        `data` and `metdata` may be updated in place.
+        `data` and `metadata` may be updated in place.
 
         Always returns a tuple of ``(template, data)`` or ``(template,
         data, metadata)``, even if unchanged.
 
-        Be aware that returning a ``(template, data, None)`` triple
-        will be interpreted as using the legacy API and will indicate
-        that the template is a legacy Genshi template. The same will
-        happen if the third value of the tuple is a string.
-
-        The `method` last parameter is deprecated and is now supposed
-        to be passed as `'method'` in the `metadata` dict. Even
-        better, use directly the `'text'` indication (`True` or
-        `False`).
-
-        Note that `template`, `data`, `content_type` will be `None` if:
+        Note that `template`, `data`, `metadata` will be `None` if:
          - called when processing an error page
          - the default request handler did not return any result
 
@@ -173,43 +164,14 @@ class IRequestFilter(Interface):
            implementation's signature.
 
         :Since 1.3.2: Genshi templates are still supported, and if
-           `process_request` used the old API (`(template, data,
+           `process_request` uses the old API (`(template, data,
            content_type)`), the `metadata` parameter passed to
            `post_process_request` will actually be the `content_type`
-           value (`String` or `None`).  This support for the old API
-           will be removed in Trac 1.5.1, in which `metadata` will
-           always be a `dict` or `None`.
+           value (`String` or `None`).
 
-        """
+        :Since 1.5.1.: Genshi templates are no longer supported. `metadata`
+            is always a `dict` or `None`.
 
-
-class ITemplateStreamFilter(Interface):
-    """Transform the generated content by filtering the Genshi event stream
-    generated by the template, prior to its serialization.
-
-    :deprecated: the Genshi template filtering concept doesn't apply
-                 anymore to Jinja2 templates, please consider
-                 converting your plugins to perform browser-side
-                 modifications of the rendered page using JavaScript.
-                 This interface will be removed in Trac 1.5.1.
-
-    See :teo:`TracDev/PortingFromGenshiToJinja#ReplacingITemplateStreamFilter`
-    for details.
-
-    """
-
-    def filter_stream(req, method, filename, stream, data):
-        """Return a filtered Genshi event stream, or the original unfiltered
-        stream if no match.
-
-        `req` is the current request object, `method` is the Genshi render
-        method (xml, xhtml or text), `filename` is the filename of the template
-        to be rendered, `stream` is the event stream and `data` is the data for
-        the current template.
-
-        See the Genshi_ documentation for more information.
-
-        .. _Genshi: http://genshi.edgewall.org/wiki/Documentation/filters.html
         """
 
 
@@ -225,14 +187,12 @@ class TracNotImplementedError(TracError, NotImplementedError):
     title = N_("Not Implemented Error")
 
 
-HTTP_STATUS = {code: reason.title()
+HTTP_STATUS = {int(code): reason.title()
                for code, (reason, description)
                in BaseHTTPRequestHandler.responses.items()}
 
 
-class HTTPException(TracBaseError):
-
-    __metaclass__ = ABCMeta
+class HTTPException(TracBaseError, metaclass=ABCMeta):
 
     code = None
 
@@ -245,7 +205,7 @@ class HTTPException(TracBaseError):
             self.detail = detail
         if args:
             self.detail = self.detail % args
-        arg = u'%s %s (%s)' % (self.code, self.reason, to_unicode(self.detail))
+        arg = '%s %s (%s)' % (self.code, self.reason, to_unicode(self.detail))
         super(HTTPException, self).__init__(arg)
 
     @property
@@ -303,6 +263,10 @@ del code, exc_name
 class _FieldStorage(cgi.FieldStorage):
     """Our own version of cgi.FieldStorage, with tweaks."""
 
+    def make_file(self):
+        # We always use binary mode even if filename parameter is missing
+        return tempfile.TemporaryFile('wb+')
+
     def read_multi(self, *args, **kwargs):
         try:
             cgi.FieldStorage.read_multi(self, *args, **kwargs)
@@ -467,8 +431,11 @@ class _RequestArgs(dict):
         if hasattr(upload, 'file'):
             fileobj = upload.file
             if hasattr(fileobj, 'fileno'):
-                size = os.fstat(fileobj.fileno())[6]
-            else:
+                try:
+                    size = os.fstat(fileobj.fileno())[6]
+                except io.UnsupportedOperation:
+                    size = None
+            if size is None:
                 fileobj.seek(0, 2)  # seek to end of file
                 size = fileobj.tell()
                 fileobj.seek(0)
@@ -490,12 +457,12 @@ def parse_arg_list(query_string):
             (name, value) = nv
         else:
             (name, value) = (nv[0], empty)
-        name = urllib.unquote(name.replace('+', ' '))
-        if isinstance(name, str):
-            name = unicode(name, 'utf-8')
-        value = urllib.unquote(value.replace('+', ' '))
-        if isinstance(value, str):
-            value = unicode(value, 'utf-8')
+        name = urllib.parse.unquote(name.replace('+', ' '))
+        if isinstance(name, bytes):
+            name = str(name, 'utf-8')
+        value = urllib.parse.unquote(value.replace('+', ' '))
+        if isinstance(value, bytes):
+            value = str(value, 'utf-8')
         args.append((name, value))
     return args
 
@@ -638,8 +605,13 @@ class Request(object):
     def path_info(self):
         """Path inside the application"""
         path_info = self.environ.get('PATH_INFO', '')
+        if isinstance(path_info, str):
+            # According to PEP 3333, the value is decoded by iso-8859-1
+            # encoding when it is a unicode string. However, we need
+            # decoded unicode string by utf-8 encoding.
+            path_info = path_info.encode('iso-8859-1')
         try:
-            return unicode(path_info, 'utf-8')
+            return str(path_info, 'utf-8')
         except UnicodeDecodeError:
             raise HTTPNotFound(_("Invalid URL encoding (was %(path_info)r)",
                                  path_info=path_info))
@@ -712,12 +684,12 @@ class Request(object):
     def send_header(self, name, value):
         """Send the response header with the specified name and value.
 
-        `value` must either be an `unicode` string or can be converted to one
+        `value` must either be a `str` string or can be converted to one
         (e.g. numbers, ...)
         """
         if name.lower() == 'content-type':
             self._content_type = value.split(';', 1)[0]
-        self._outheaders.append((name, unicode(value).encode('utf-8')))
+        self._outheaders.append((name, str(value)))
 
     def end_headers(self, exc_info=None):
         """Must be called after all headers have been sent and before the
@@ -749,7 +721,7 @@ class Request(object):
         if isinstance(extra, list):
             m = hashlib.sha1()
             for elt in extra:
-                m.update(repr(elt))
+                m.update(repr(elt).encode('utf-8'))
             extra = m.hexdigest()
         etag = 'W/"%s/%s/%s"' % (self.authname, http_date(datetime), extra)
         inm = self.get_header('If-None-Match')
@@ -782,8 +754,9 @@ class Request(object):
         self.send_response(status)
         if not url.startswith(('http://', 'https://')):
             # Make sure the URL is absolute
-            scheme, host = urlparse.urlparse(self.base_url)[:2]
-            url = urlparse.urlunparse((scheme, host, url, None, None, None))
+            scheme, host = urllib.parse.urlparse(self.base_url)[:2]
+            url = urllib.parse.urlunparse((scheme, host, url, None, None,
+                                           None))
 
         # Workaround #10382, IE6-IE9 bug when post and redirect with hash
         if status == 303 and '#' in url:
@@ -805,48 +778,8 @@ class Request(object):
     def send(self, content, content_type='text/html', status=200):
         self._send(content, content_type, status)
 
-    def send_error(self, exc_info, template='error.html',
-                   content_type='text/html', status=500, env=None, data={}):
-        """Render error page.
-
-        The rendered content can also be passed to the method, when the
-        method is invoked as:
-        `send_error(exc_info, content, content_type='text/html', status=500)`
-
-        :since 1.3.3: the method that renders an error page is deprecated
-            in favor of the method that sends rendered content. The method
-            that renders an error page will be removed in 1.5.1.
-        """
-        try:
-            if isinstance(template, basestring) and template.endswith('.html'):
-                if env:
-                    from trac.web.chrome import Chrome, add_stylesheet
-                    add_stylesheet(self, 'common/css/code.css')
-                    metadata = {'content_type': 'text/html'}
-                    try:
-                        content = Chrome(env).render_template(self, template,
-                                                              data, metadata)
-                    except Exception:
-                        # second chance rendering, in "safe" mode
-                        data['trac_error_rendering'] = True
-                        content = Chrome(env).render_template(self, template,
-                                                              data, metadata)
-                else:
-                    content_type = 'text/plain'
-                    content = '%s\n\n%s: %s' % (data.get('title'),
-                                                data.get('type'),
-                                                data.get('message'))
-            else:
-                # send_error(self, exc_info, content, content_type='text/html',
-                #            status=500)
-                content = template
-        except Exception: # failed to render
-            content = get_last_traceback()
-            content_type = 'text/plain'
-
-        if isinstance(content, unicode):
-            content = content.encode('utf-8')
-
+    def send_error(self, exc_info, content, content_type='text/html',
+                   status=500):
         self._outheaders = []
         self._send(content, content_type, status, exc_info)
 
@@ -918,8 +851,8 @@ class Request(object):
     def write(self, data):
         """Write the given data to the response body.
 
-        *data* **must** be a `str` string or an iterable instance
-        which iterates `str` strings, encoded with the charset which
+        *data* **must** be a `bytes` string or an iterable instance
+        which iterates `bytes` strings, encoded with the charset which
         has been specified in the ``'Content-Type'`` header or UTF-8
         otherwise.
 
@@ -935,21 +868,21 @@ class Request(object):
             bufsize = 0
             buf = []
             buf_append = buf.append
-            if isinstance(data, basestring):
+            if isinstance(data, bytes):
                 data = [data]
             for chunk in data:
-                if isinstance(chunk, unicode):
-                    raise ValueError("Can't send unicode content")
+                if isinstance(chunk, str):
+                    raise ValueError("Can't send str content")
                 if not chunk:
                     continue
                 bufsize += len(chunk)
                 buf_append(chunk)
                 if bufsize >= chunk_size:
-                    self._write(''.join(buf))
+                    self._write(b''.join(buf))
                     bufsize = 0
                     buf[:] = ()
             if bufsize > 0:
-                self._write(''.join(buf))
+                self._write(b''.join(buf))
         except IOError as e:
             if is_client_disconnect_exception(e):
                 raise RequestDone
@@ -974,7 +907,7 @@ class Request(object):
         self.send_header('Cache-Control', 'must-revalidate')
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.send_header('Content-Type', content_type + ';charset=utf-8')
-        if isinstance(content, basestring):
+        if isinstance(content, str):
             self.send_header('Content-Length', len(content))
         self.end_headers(exc_info)
 
@@ -1004,7 +937,10 @@ class Request(object):
             qs_on_post = self.environ.pop('QUERY_STRING', '')
         try:
             fs = _FieldStorage(fp, environ=self.environ,
-                               keep_blank_values=True)
+                               keep_blank_values=True, errors='strict')
+        except UnicodeDecodeError as e:
+            raise HTTPBadRequest(_("Invalid encoding in form data: %(msg)s",
+                                   msg=exception_to_unicode(e)))
         except IOError as e:
             if is_client_disconnect_exception(e):
                 raise HTTPBadRequest(
@@ -1022,19 +958,11 @@ class Request(object):
         for value in fs.list or ():
             name = value.name
             raise_if_null_bytes(name)
-            try:
-                if name is not None:
-                    name = unicode(name, 'utf-8')
-                if value.filename:
-                    raise_if_null_bytes(value.filename)
-                else:
-                    value = value.value
-                    raise_if_null_bytes(value)
-                    value = unicode(value, 'utf-8')
-            except UnicodeDecodeError as e:
-                raise HTTPBadRequest(
-                    _("Invalid encoding in form data: %(msg)s",
-                      msg=exception_to_unicode(e)))
+            if value.filename:
+                raise_if_null_bytes(value.filename)
+            else:
+                value = value.value
+                raise_if_null_bytes(value)
             args.append((name, value))
         return args
 
@@ -1085,8 +1013,8 @@ class Request(object):
                 host = '%s:%d' % (self.server_name, self.server_port)
             else:
                 host = self.server_name
-        return urlparse.urlunparse((self.scheme, host, self.base_path, None,
-                                    None, None))
+        return urllib.parse.urlunparse((self.scheme, host, self.base_path,
+                                        None, None, None))
 
     def _send_configurable_headers(self):
         sent_headers = [name.lower() for name, val in self._outheaders]
@@ -1103,7 +1031,7 @@ class Request(object):
                            .replace(',', '%3C')
             self.outcookie[name]['path'] = path
 
-        cookies = to_unicode(self.outcookie.output(header='')).encode('utf-8')
+        cookies = to_unicode(self.outcookie.output(header=''))
         for cookie in cookies.splitlines():
             self._outheaders.append(('Set-Cookie', cookie.strip()))
 
