@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2019 Edgewall Software
+# Copyright (C) 2003-2020 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -102,7 +102,7 @@ class WikiPage(object):
         if self.name is None:
             name = self.name
         else:
-            name = u'%s@%s' % (self.name, self.version)
+            name = '%s@%s' % (self.name, self.version)
         return '<%s %r>' % (self.__class__.__name__, name)
 
     exists = property(lambda self: self.version > 0)
@@ -144,7 +144,7 @@ class WikiPage(object):
                 if hasattr(listener, 'wiki_page_version_deleted'):
                     listener.wiki_page_version_deleted(self)
 
-    def save(self, author, comment, t=None):
+    def save(self, author, comment, t=None, replace=False):
         """Save a new version of a page."""
         if not validate_page_name(self.name):
             raise TracError(_("Invalid Wiki page name '%(name)s'",
@@ -157,12 +157,17 @@ class WikiPage(object):
 
         with self.env.db_transaction as db:
             if new_text:
-                db("""INSERT INTO wiki (name, version, time, author,
-                                        text, comment, readonly)
-                      VALUES (%s,%s,%s,%s,%s,%s,%s)
-                      """, (self.name, self.version + 1, to_utimestamp(t),
-                            author, self.text, comment, self.readonly))
-                self.version += 1
+                if replace and self.version != 0:
+                    db("""
+                        UPDATE wiki SET text=%s WHERE name=%s AND version=%s
+                        """, (self.text, self.name, self.version))
+                else:
+                    self.version += 1
+                    db("""INSERT INTO wiki
+                           (name,version,time,author,text,comment,readonly)
+                          VALUES (%s,%s,%s,%s,%s,%s,%s)
+                          """, (self.name, self.version, to_utimestamp(t),
+                                author, self.text, comment, self.readonly))
             else:
                 db("UPDATE wiki SET readonly=%s WHERE name=%s",
                    (self.readonly, self.name))
@@ -193,6 +198,12 @@ class WikiPage(object):
         if not self.exists:
             raise TracError(_("Cannot rename non-existent page"))
 
+        if not new_name:
+            raise TracError(_("A new name is mandatory for a rename."))
+
+        if self.name == new_name:
+            raise TracError(_("Page name is unchanged."))
+
         if not validate_page_name(new_name):
             raise TracError(_("Invalid Wiki page name '%(name)s'",
                               name=new_name))
@@ -201,8 +212,8 @@ class WikiPage(object):
         with self.env.db_transaction as db:
             new_page = WikiPage(self.env, new_name)
             if new_page.exists:
-                raise TracError(_("Can't rename to existing %(name)s page.",
-                                  name=new_name))
+                raise TracError(_("The page '%(name)s' already exists.",
+                                   name=new_name))
 
             db("UPDATE wiki SET name=%s WHERE name=%s", (new_name, old_name))
             # Invalidate page name cache
