@@ -16,8 +16,6 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Matthew Good <trac@matt-good.net>
 
-from __future__ import print_function
-
 import cgi
 import fnmatch
 from functools import partial
@@ -30,7 +28,7 @@ from pprint import pformat, pprint
 import re
 import sys
 import traceback
-from urlparse import urlparse
+from urllib.parse import urlparse
 
 from jinja2 import FileSystemLoader
 
@@ -44,8 +42,7 @@ from trac.loader import get_plugin_info, match_plugins_to_frames
 from trac.perm import PermissionCache, PermissionError
 from trac.resource import ResourceNotFound
 from trac.util import arity, get_frame_info, get_last_traceback, hex_entropy, \
-                      lazy, read_file, safe_repr, translation, \
-                      warn_setuptools_issue
+                      lazy, read_file, safe_repr, translation
 from trac.util.concurrency import get_thread_id
 from trac.util.datefmt import format_datetime, localtz, timezone, user_time
 from trac.util.html import tag, valid_html_bytes
@@ -170,7 +167,8 @@ class RequestDispatcher(Component):
     configurable_headers = ConfigSection('http-headers', """
         Headers to be added to the HTTP request. (''since 1.2.3'')
 
-        The header name must conform to RFC7230 and the following
+        The header name must conform to
+        [https://tools.ietf.org/html/rfc7230 RFC7230] and the following
         reserved names are not allowed: content-type, content-length,
         location, etag, pragma, cache-control, expires.
         """)
@@ -249,27 +247,23 @@ class RequestDispatcher(Component):
             # Process the request and render the template
             resp = chosen_handler.process_request(req)
             if resp:
-                resp = self._post_process_request(req, *resp)
-                template, data, metadata, method = resp
+                template, data, metadata = \
+                    self._post_process_request(req, *resp)
                 if 'hdfdump' in req.args:
                     req.perm.require('TRAC_ADMIN')
                     # debugging helper - no need to render first
-                    out = io.BytesIO()
-                    pprint({'template': template,
-                            'metadata': metadata,
-                            'data': data}, out)
-                    req.send(out.getvalue(), 'text/plain')
+                    with io.TextIOWrapper(io.BytesIO(), encoding='utf-8',
+                                          newline='\n',
+                                          write_through=True) as out:
+                        pprint({'template': template,
+                                'metadata': metadata,
+                                'data': data}, out)
+                        out = out.buffer.getvalue()
+                    req.send(out, 'text/plain')
                 self.log.debug("Rendering response with template %s", template)
-                iterable = chrome.use_chunked_encoding
-                if isinstance(metadata, dict):
-                    iterable = metadata.setdefault('iterable', iterable)
-                    content_type = metadata.get('content_type')
-                else:
-                    content_type = metadata
-                output = chrome.render_template(req, template, data, metadata,
-                                                iterable=iterable,
-                                                method=method)
-                # TODO (1.5.1) remove iterable and method parameters
+                metadata.setdefault('iterable', chrome.use_chunked_encoding)
+                content_type = metadata.get('content_type')
+                output = chrome.render_template(req, template, data, metadata)
                 req.send(output, content_type or 'text/html')
             else:
                 self.log.debug("Empty or no response from handler. "
@@ -293,18 +287,18 @@ class RequestDispatcher(Component):
                                    " request: %s",
                                    exception_to_unicode(e2, traceback=True))
             if isinstance(e, PermissionError):
-                raise HTTPForbidden(e)
+                raise HTTPForbidden(e) from e
             if isinstance(e, ResourceNotFound):
-                raise HTTPNotFound(e)
+                raise HTTPNotFound(e) from e
             if isinstance(e, NotImplementedError):
                 tb = traceback.extract_tb(err[2])[-1]
                 self.log.warning("%s caught from %s:%d in %s: %s",
                                  e.__class__.__name__, tb[0], tb[1], tb[2],
                                  to_unicode(e) or "(no message)")
-                raise HTTPInternalServerError(TracNotImplementedError(e))
+                raise HTTPInternalServerError(TracNotImplementedError(e)) from e
             if isinstance(e, TracError):
-                raise HTTPInternalServerError(e)
-            raise err[0], err[1], err[2]
+                raise HTTPInternalServerError(e) from e
+            raise e
 
     # ITemplateProvider methods
 
@@ -429,7 +423,7 @@ class RequestDispatcher(Component):
     def _xsendfile_header(self):
         header = self.xsendfile_header.strip()
         if Request.is_valid_header(header):
-            return to_utf8(header)
+            return header
         else:
             if not self._warn_xsendfile_header:
                 self._warn_xsendfile_header = True
@@ -463,43 +457,21 @@ class RequestDispatcher(Component):
         return chosen_handler
 
     def _post_process_request(self, req, *args):
-        resp = args
-        # `metadata` and the backward compatibility `method` are
-        # optional in IRequestHandler's response. If not specified,
-        # the default value is appended to response.
         metadata = {}
-        method = None
-        if len(resp) == 2:
-            resp += (metadata, None)
-        elif len(resp) == 3:
-            metadata = resp[2]
-            resp += (None,)
-        elif len(resp) == 4:
-            metadata = resp[2]
-            method = resp[3]
-        if method and isinstance(metadata, dict):
-            metadata['method'] = method
-        nbargs = len(resp)
+        resp = args
+        if len(args) == 3:
+            metadata = args[2]
+        elif len(args) == 2:
+            resp += (metadata,)
+        elif len(args) == 0:
+            resp = (None,) * 3
         for f in reversed(self.filters):
-            # As the arity of `post_process_request` has changed since
-            # Trac 0.10, only filters with same arity gets passed real values.
-            # Errors will call all filters with None arguments,
-            # and results will not be not saved.
-            extra_arg_count = arity(f.post_process_request) - 1
-            if extra_arg_count == nbargs:
-                resp = f.post_process_request(req, *resp)
-            elif extra_arg_count == nbargs - 1:
-                # IRequestFilters may modify the `method`, but the `method`
-                # is forwarded when not accepted by the IRequestFilter.
-                method = resp[-1]
-                resp = f.post_process_request(req, *resp[:-1])
-                resp += (method,)
-            elif nbargs == 0:
-                f.post_process_request(req, *(None,) * extra_arg_count)
+            resp = f.post_process_request(req, *resp)
+            if len(resp) == 2:
+                resp += (metadata,)
         return resp
 
 
-_warn_setuptools = False
 _slashes_re = re.compile(r'/+')
 
 def dispatch_request(environ, start_response):
@@ -509,11 +481,6 @@ def dispatch_request(environ, start_response):
     :param start_response: the WSGI callback for starting the response
     """
 
-    global _warn_setuptools
-    if _warn_setuptools is False:
-        _warn_setuptools = True
-        warn_setuptools_issue(out=environ.get('wsgi.errors'))
-
     # SCRIPT_URL is an Apache var containing the URL before URL rewriting
     # has been applied, so we can use it to reconstruct logical SCRIPT_NAME
     script_url = environ.get('SCRIPT_URL')
@@ -568,7 +535,9 @@ def dispatch_request(environ, start_response):
             # the remaining path in the `PATH_INFO` variable.
             script_name = environ.get('SCRIPT_NAME', '')
             try:
-                script_name = unicode(script_name, 'utf-8')
+                if isinstance(script_name, str):
+                    script_name = script_name.encode('iso-8859-1')  # PEP 3333
+                script_name = str(script_name, 'utf-8')
             except UnicodeDecodeError:
                 errmsg = 'Invalid URL encoding (was %r)' % script_name
             else:
@@ -585,6 +554,7 @@ def dispatch_request(environ, start_response):
                     errmsg = 'Environment not found'
 
             if errmsg:
+                errmsg = errmsg.encode('utf-8')
                 start_response('404 Not Found',
                                [('Content-Type', 'text/plain'),
                                 ('Content-Length', str(len(errmsg)))])
@@ -691,7 +661,7 @@ def _send_error(req, exc_info, template='error.html', content_type='text/html',
                                     data.get('type'),
                                     data.get('message'))
 
-    if isinstance(content, unicode):
+    if isinstance(content, str):
         content = content.encode('utf-8')
 
     try:
@@ -739,8 +709,8 @@ def send_internal_error(env, req, exc_info):
         if env:
             plugins = [p for p in get_plugin_info(env)
                        if any(c['enabled']
-                              for m in p['modules'].itervalues()
-                              for c in m['components'].itervalues())]
+                              for m in p['modules'].values()
+                              for c in m['components'].values())]
             match_plugins_to_frames(plugins, frames)
 
             # Identify the tracker where the bug should be reported
@@ -770,10 +740,10 @@ def send_internal_error(env, req, exc_info):
             enabled_plugins = "".join("|| '''`%s`''' || `%s` ||\n"
                                       % (p['name'], p['version'] or _('N/A'))
                                       for p in plugins)
-            files = Chrome(env).get_interface_customization_files().items()
+            files = Chrome(env).get_interface_customization_files()
             interface_files = "".join("|| **%s** || %s ||\n"
                                       % (k, ", ".join("`%s`" % f for f in v))
-                                      for k, v in sorted(files))
+                                      for k, v in sorted(files.items()))
         else:
             sys_info = _("''System information not available''\n")
             enabled_plugins = _("''Plugin information not available''\n")
