@@ -42,10 +42,7 @@ class DefaultWikiChecker(Formatter):
     def handle_match(self, fullmatch):
         rv = self.__super.handle_match(fullmatch)
         if rv:
-            if not isinstance(rv, basestring):
-                text = unicode(rv)
-            else:
-                text = rv
+            text = str(rv) if not isinstance(rv, str) else rv
             if text.startswith('<a ') and text.endswith('</a>') and \
                     'class="missing ' in text:
                 self.__marks.append((fullmatch.start(0), fullmatch.end(0)))
@@ -98,7 +95,7 @@ class LinesIterator(object):
         self.idx = 0
         self.current = None
 
-    def next(self):
+    def __next__(self):
         idx = self.idx
         if self.current is not None:
             self.formatter.next_callback(self.current, idx)
@@ -142,7 +139,7 @@ re_box_processor = re.compile(r'{{{#!box[^\}]+}}}\s*\r?\n?')
 
 
 def download_default_pages(names, prefix, strict):
-    from httplib import HTTPSConnection
+    from http.client import HTTPSConnection
     host = 'trac.edgewall.org'
     if prefix and not prefix.endswith('/'):
         prefix += '/'
@@ -160,8 +157,10 @@ def download_default_pages(names, prefix, strict):
                 conn.request('GET', '/wiki/%s?format=txt' % name)
                 response = conn.getresponse()
                 content = response.read()
+            content = str(content, 'utf-8')
             if response.status == 200 and content:
-                with open('trac/wiki/default-pages/' + name, 'w') as f:
+                with open('trac/wiki/default-pages/' + name, 'w',
+                          encoding='utf-8') as f:
                     if not strict:
                         content = re_box_processor.sub('', content)
                     lines = content.replace('\r\n', '\n').splitlines(True)
