@@ -101,7 +101,7 @@ def analyze(jinja_template, only=None, quiet=False, show_ignored=False):
     """Analyzes a Jinja2 template, its control structure as well as the
     structure of the HTML.
     """
-    with open(jinja_template, 'r') as f:
+    with open(jinja_template, 'r', encoding='utf-8') as f:
         lines = f.readlines()
     line_statements, html, html_hints = scan(lines)
     issues_j = issues_h = 0
@@ -153,10 +153,11 @@ StatementTuple = namedtuple('StatementTuple',
                             ('linenum', 'indent', 'end', 'kw', 'expr', 'colon'))
 
 class Statement(StatementTuple):
-    def __init__(self, *args):
-        super(StatementTuple, self).__init__(*args)
+    def __new__(cls, *args, **kwargs):
+        self = super(Statement, cls).__new__(cls, *args, **kwargs)
         self.is_block = (self.kw in JINJA2_BLOCK_KEYWORDS or
                          self.kw == 'set' and '=' not in self.expr)
+        return self
 
 LINE_STATEMENT_RE = re.compile(r'^(\s*)%s-?(\s*)(end)?(\w+)(.*?)?(:)?$' %
                                LINE_STATEMENT_PREFIX)
@@ -176,13 +177,14 @@ def scan(lines):
     def process_multiline_expr(expr, open_parens=0):
         open_parens += count_parens(expr)
         if open_parens:
-            linenum, line = lines.next()
+            linenum, line = get_line()
             m = STATEMENT_RE.match(line)
             line_statements.append(
                 Statement(linenum, len(m.group(1)), '', '', m.group(2), ''))
             process_multiline_expr(line.rstrip(), open_parens)
 
     lines = iter(enumerate(lines, 1))
+    get_line = lambda: next(lines)
     line_statements = []
     html = []
     html_hints = []
@@ -193,7 +195,7 @@ def scan(lines):
     try:
         comment_start = -1 # not in a comment
         html_start = start_idx = end_idx = 0
-        linenum, line = lines.next()
+        linenum, line = get_line()
         html_line = []
         while True:
             # skip empty lines
@@ -241,8 +243,8 @@ def scan(lines):
                                 process_multiline_expr(expr)
                             else:
                                 html_line = line
-            html.append((linenum, ''.join(html_line).rstrip() + '\n'))
-            linenum, line = lines.next()
+            html.append((linenum, ''.join(html_line).rstrip()))
+            linenum, line = get_line()
             html_line = []
             html_start = start_idx = end_idx = 0
     except StopIteration:
@@ -372,7 +374,7 @@ def check_html(filename, html_lines, html_hints, quiet, show_ignored):
     try:
         # lxml will try to convert the URL to unicode by itself,
         # this won't work for non-ascii URLs, so help him
-        etree.parse(io.BytesIO(page), base_url='.') #  base_url ??
+        etree.parse(io.StringIO(page), base_url='.') #  base_url ??
         if not quiet:
             for lineinfo in html_lines:
                 print('%5d %s' % lineinfo),
@@ -453,7 +455,7 @@ def remove_jinja_exprs(linenum, line, opened_braces):
             normalized += attr
         pos = end + 1
     if pos < len(line):
-        normalized += line[pos:-1]
+        normalized += line[pos:]
     return normalized, opened_braces
 
 if __name__ == '__main__':
